| package |
package := Package name: 'FixedDecimalEdit'.
package paxVersion: 0;
	basicComment: 'FixedDecimalEdit is a view for decimal numbers.

This package add a resource view named ''Fixed decimal'' to NumberPresenter class.
In the ViewComposer, Resource toolbox you can select this resource and use it in any presenter.
The Fixed decimal view has a property (Aspect) ''decimalDigits''. This aspect set the size of decimal places in the presenter.
You can put into the presenter a TypeConverter. This package add two type converters: FloatToText and FloatToTextWithSeparators (for Locale printing).  
--------------------------------------------------------------
This Goodie is free. For more goodies or support:
Please visit www.Smalltalking.net
e-mail address: fxgallego@sinectis.com.ar
--------------------------------------------------------------
'.

package basicPackageVersion: ''.

"Add the package scripts"
package basicScriptAt: #postinstall put: 'NumberPresenter addView: FixedDecimalEdit asResource: ''Fixed decimal'''.

"Add the class names, loose method names, global names, resource names"
package classNames
	add: #FixedDecimalEdit;
	add: #FloatToText;
	add: #FloatToTextWithSeparators;
	yourself.

package methodNames
	add: #Float -> #printLocaleOn:decimalPlaces:;
	add: #Float -> #printWithSeparator:on:decimalPlaces:;
	add: #Integer -> #asStringWithSeparator:;
	add: 'Number class' -> #fromDisplayString:;
	yourself.

package globalNames
	yourself.

package resourceNames
	yourself.

"Binary Global Names"
package binaryGlobalNames: (Set new
	yourself).
"Resource Names"
package allResourceNames: (Set new
	yourself).

"Add the prerequisite names"
package setPrerequisites: (IdentitySet new
	add: 'Development System';
	add: 'Dolphin';
	yourself).

package!

"Class Definitions"!

NumberToText subclass: #FloatToText
	instanceVariableNames: 'decimalPlaces'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
FloatToText subclass: #FloatToTextWithSeparators
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
TextEdit subclass: #FixedDecimalEdit
	instanceVariableNames: 'decimalDigits'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!
"Loose Methods"!

!Float methodsFor!

printLocaleOn: aStream decimalPlaces: anInteger
	"Append the printed Locale representation of the receiver to aStream "
	
	| decimalSeparator thousandSeparator |
	decimalSeparator := Locale default decimalSeparator first.
	thousandSeparator := $,.
	decimalSeparator = $.
			ifFalse: [ thousandSeparator := $. ].
	self printWithSeparator: thousandSeparator on: aStream decimalPlaces: anInteger
			
!

printWithSeparator: aCharacter on: aStream decimalPlaces: anInteger
	"Append the printed representation of the receiver to aStream separated by aCharacter and
	anInteger decimal places"

	| int strm |
	int := self integerPart asInteger.
	strm := String writeStream.
	aStream nextPutAll: (int asStringWithSeparator: aCharacter).
	self printOn: strm decimalPlaces: anInteger.
	strm := strm contents subStrings: $..
	aCharacter = $.
		ifTrue: [ aStream nextPut: $, ] ifFalse: [ aStream nextPut: $. ].
	aStream nextPutAll: strm second
	! !
!Float categoriesFor: #printLocaleOn:decimalPlaces:!printing!public! !
!Float categoriesFor: #printWithSeparator:on:decimalPlaces:!printing!public! !

!Integer methodsFor!

asStringWithSeparator: aCharacter
	"Answer the receiver as String separated by aCharacter"

	| digits stream |
	digits := self abs printString.
	stream := String writeStream.
	self sign = -1 ifTrue: [stream nextPut: $-].
	1 to: digits size do: [ :i | stream nextPut: (digits at: i).
					(i < digits size and: [(i - digits size) \\ 3 = 0])
					ifTrue: [stream nextPut: aCharacter]].
	^stream contents
! !
!Integer categoriesFor: #asStringWithSeparator:!*-unclassified!public! !

!Number class methodsFor!

fromDisplayString: aString
	"Instantiate a new instance of the receiver from the <readableString> argument.
	Support for Locale format with ',' and '.' characters."

	| decimalSeparator newString |
	decimalSeparator := Locale default decimalSeparator first.
	decimalSeparator = $.
			ifTrue: [ newString := aString reject: [ :each | each = $, ] ]
			ifFalse: [ | i | newString := aString reject: [ :each | each = $. ].
					   i := newString indexOf: $, ifAbsent: [ nil ].
					   i notNil ifTrue: [ newString at: i put: $. ] ].
	^self readFrom: newString readStream.! !
!Number class categoriesFor: #fromDisplayString:!instance creation!public! !

"End of package definition"!



FloatToText comment: ''!

FloatToText guid: (GUID fromString: '{7E410260-19AB-11D6-A248-004352C10001}')!

!FloatToText categoriesForClass!MVP-Type Converters-Text! !
!FloatToText methodsFor!

decimalPlaces
	"Answer the value of the receiver's ''decimalPlaces'' instance variable."

	decimalPlaces == nil
		ifTrue: [ decimalPlaces := 2 ].
	^decimalPlaces!

decimalPlaces: anObject
	"Set the value of the receiver's ''decimalPlaces'' instance variable to the argument, anObject."

	decimalPlaces := anObject isInteger ifFalse: [ 0 ] ifTrue: [ anObject ]!

leftToRight: aNumber
	"Answers the result of converting aNumber to a String"

	| stream |
	stream := String writeStream.
	aNumber asFloat printOn: stream decimalPlaces: self decimalPlaces.
	^stream contents 
	!

rightToLeft: aString
	"Answers the result of converting aString to a Number."

	^(Number fromString: aString) asFloat
! !
!FloatToText categoriesFor: #decimalPlaces!accessing!public! !
!FloatToText categoriesFor: #decimalPlaces:!accessing!public! !
!FloatToText categoriesFor: #leftToRight:!operations!public! !
!FloatToText categoriesFor: #rightToLeft:!*-unclassified!public! !

!FloatToText class methodsFor!

publishedAspectsOfInstances
	"Answer a Set of AspectDescriptors that describe the aspects published
	by instances instances of the receiver" 

	^super publishedAspectsOfInstances
		add: (Aspect name: #decimalPlaces);
		yourself
! !
!FloatToText class categoriesFor: #publishedAspectsOfInstances!*-unclassified!public! !



FloatToTextWithSeparators comment: ''!

FloatToTextWithSeparators guid: (GUID fromString: '{295001A0-25F6-11D6-A248-E07D53C10801}')!

!FloatToTextWithSeparators categoriesForClass!MVP-Type Converters-Text! !
!FloatToTextWithSeparators methodsFor!

leftToRight: aNumber
	"Answers the result of converting aNumber to a String in Locale format "

	| stream |
	stream := String writeStream.
	aNumber asFloat printLocaleOn: stream decimalPlaces: self decimalPlaces.
	^stream contents 
	!

rightToLeft: aString
	"Answers the result of converting aString in number Locale format to a Float."

	^(Number fromDisplayString: aString) asFloat
! !
!FloatToTextWithSeparators categoriesFor: #leftToRight:!operations!public! !
!FloatToTextWithSeparators categoriesFor: #rightToLeft:!*-unclassified!public! !



FixedDecimalEdit comment: ''!

FixedDecimalEdit guid: (GUID fromString: '{52B551A0-0F60-11D6-A248-807F53C10801}')!

!FixedDecimalEdit categoriesForClass!Unclassified! !
!FixedDecimalEdit methodsFor!

decimalDigits
	"Answer the max number of decimal digits holded by the receiver"

	decimalDigits == nil
			ifTrue: [ decimalDigits := 2 ].
	^decimalDigits!

decimalDigits: anInteger
	"Set the max number of decimal digits holded by the receiver"

	decimalDigits := anInteger isInteger ifFalse: [ 0 ] ifTrue: [ anInteger ] 

!

decimalSeparator
	"Private - Answer the decimal separator Character used by the default Locale"

	^Locale default decimalSeparator first!

isDecimalSeparator: aCharacter
	"Private - Answer true if aCharacter is a valid decimal point character and is in a valid position,
	 else answer false"

	^((self decimalSeparator == aCharacter) and: [ self isValidSeparatorPosition ])!

isIntegerCharacter: aCharacter
	"Private - Answer true if aCharacter is valid for a decimal point number, 
	 else answer false"

	| separatorIndex |
	separatorIndex := self displayValue indexOf: self decimalSeparator. 
	^(aCharacter isDigit and:
		 [ (self displayValue includes: self decimalSeparator)
				ifTrue: [ ^((self displayValue size - separatorIndex < self decimalDigits)
						or: [ self caretPosition <= separatorIndex ])  ]
				ifFalse: [ ^true ] ])  !

isValidCharacter: aCharacter
	"Private - Answer true if aCharacter is acceptable for a Float, else answer false"

	^((self isIntegerCharacter: aCharacter) or: [ self isDecimalSeparator: aCharacter ])!

isValidKeyTyped: aKeyEvent
	"Private - Answer true if aKeyEvent is valid for the receiver view, else answer false"

	((aKeyEvent code == VK_BACK) or: [ aKeyEvent code == VK_RETURN ])
		ifFalse: [ ^self isValidCharacter: aKeyEvent code asCharacter ].
	
	^true
	!

isValidSeparatorPosition
	"Private - Answer true if the decimalSeparator is well located and it's the only one"

	(self displayValue includes: self decimalSeparator)
		ifFalse: [ ^self caretPosition ~= 1 ].
	^false!

onKeyTyped: aKeyEvent
	"Default handler for the receiver's view receiving a key entry."

	(self isValidKeyTyped: aKeyEvent)
			ifTrue: [ ^super onKeyTyped: aKeyEvent ].
	^false! !
!FixedDecimalEdit categoriesFor: #decimalDigits!*-unclassified!public! !
!FixedDecimalEdit categoriesFor: #decimalDigits:!*-unclassified!public! !
!FixedDecimalEdit categoriesFor: #decimalSeparator!*-unclassified!private! !
!FixedDecimalEdit categoriesFor: #isDecimalSeparator:!*-unclassified!private! !
!FixedDecimalEdit categoriesFor: #isIntegerCharacter:!*-unclassified!private! !
!FixedDecimalEdit categoriesFor: #isValidCharacter:!*-unclassified!private! !
!FixedDecimalEdit categoriesFor: #isValidKeyTyped:!*-unclassified!private! !
!FixedDecimalEdit categoriesFor: #isValidSeparatorPosition!*-unclassified!private! !
!FixedDecimalEdit categoriesFor: #onKeyTyped:!*-unclassified!public! !

!FixedDecimalEdit class methodsFor!

publishedAspectsOfInstances
	"Answer a Set of the aspects published by  instances of the receiver"

	^super publishedAspectsOfInstances 
		add: (Aspect integer: #decimalDigits);
		yourself! !
!FixedDecimalEdit class categoriesFor: #publishedAspectsOfInstances!constants!development!public! !

 
"Binary Globals"!

"Resources"!

